-- Creature Cards
-- By Tim Sandford, 2019
-- Comments to timsandford42@hotmail.com
function love.run()  -- Main function
  gVERSION, gPATCH, gREVISION, gCODENAME = love.getVersion()
  if gVERSION >=  11 then  -- Colour fix for 11.0 onwards
    local oldSetColor = love.graphics.setColor
    love.graphics.setColor = function (r, g, b, a)     
      if type(r)=="table" then
        r = r[1] / 255
        g = r[2] / 255
        b = r[3] / 255
        a = (r[4] or 255) / 255
      else
        r = r / 255
        g = g / 255
        b = b / 255
        a = (a or 255) / 255
      end
      oldSetColor(r,g,b,a)
    end
  end
  math.randomseed(os.time())  -- True randomness
  love.load(love.arg.parseGameArguments(arg), arg) -- load variables etc.
  while true do  -- Main loop
    if love.event then -- Process events
      love.event.pump()
      for name, a,b,c,d,e,f in love.event.poll() do
        if name == "quit" then
          if not love.quit or not love.quit() then
            return a
          end
        end
        love.handlers[name](a,b,c,d,e,f)
      end
    end
    love.update() -- Update
    love.graphics.origin() -- Not sure, probably reset origin?
    love.graphics.clear(love.graphics.getBackgroundColor()) -- Clears the screen
    love.draw() -- Collect draw commands
    love.graphics.present() -- Present the graphics on the screen!
    love.timer.sleep(0.001) -- This is to lower the load on the CPU, but I could edit it :)		
  end
end

function love.load(arg)
  local i, s
  io.stdout:setvbuf("no")                                         -- Print messages immediately, allows tracing bugs
  if arg[#arg] == "-debug" then require("mobdebug").start() end   -- Zerobrane debugging
  gGAMEMODE = "Main Menu"                                         -- Set gamemode
  gSCREENWIDTH = love.graphics.getWidth()                         -- Get screensize
  gSCREENHEIGHT = love.graphics.getHeight()                       -- Get screensize
  gRATIOX = gSCREENWIDTH / 800                                    -- Scale graphics to screen
  gRATIOY = gSCREENHEIGHT / 600                                   -- Scale graphics to screen
  gFADE = 255                                                     -- Fade in effect
  gFADESPEED = 10                                                 -- Speed of fade in effect
  gFPS = 60                                                       -- Frames per second
  gTIMER = love.timer.getTime()                                   -- Use timer in love.update for speed of game
  gSOUNDON = true                                                 -- Sounds on
  gMUSICON = true                                                 -- Music on
  gFLASH = 255			                                               -- flashing text transparency
  gFLASH_z = 10 			                                             -- speed of change for flashing text
  gFLASH_min = 80                                                 -- minimum transparency of anything flashing
  gFLASH_max = 255                                                -- maximum transparency of anything flashing
  
  tCONTROL = {"AI", "AI"}
  gTURN =  math.random(2)
  
  tHABITAT = {}
  tCARDS = {}
  readcards()
  --loadhabitats()
  
  sSOUND = {}                                                     -- Load sounds
  for i = 1, 99, 1 do
    if i < 10 then s = "sounds/sound0"..i..".ogg"
    else s = "sounds/sound"..i..".ogg" end
    if love.filesystem.getInfo(s, "file") then sSOUND[i] = love.audio.newSource(s, "static") end
  end
  
  if love.filesystem.getInfo("gui/cursor.png", "file") then       -- Mouse pointer
    gCURSOR = love.mouse.newCursor("gui/cursor.png", 0, 0)
    love.mouse.setCursor(gCURSOR)
  end
  love.mouse.setVisible(true)
  
  fTINY = love.graphics.newFont("fonts/font.ttf", 9)              -- tiny font
  fSMALL = love.graphics.newFont("fonts/font.ttf", 12)            -- small font
  fDEFAULT = love.graphics.newFont("fonts/font.ttf", 16)          -- default font
  fHEADING = love.graphics.newFont("fonts/font.ttf", 28)          -- heading font
  fTITLE = love.graphics.newFont("fonts/font.ttf", 40)            -- title font
  
  iICON = {}
  for i = 1, 99, 1 do
    if i < 10 then s = "gui/icon0"..i..".png"
    else s = "gui/icon"..i..".png" end
    if love.filesystem.getInfo(s, "file") then iICON[i] = love.graphics.newImage(s) end
  end
end

function readcards()
  local i = 0
  local j, s
  if love.filesystem.getInfo("data/cards.txt", "file") then
    tCARDS = {}
    for line in love.filesystem.lines("data/cards.txt") do
      if string.find(line, "ID=") then
        i = i + 1
        tCARDS[i] = {}
        tCARDS[i].ID = tonumber(string.sub(line, 4))
        tCARDS[i].Special = "None"
        tCARDS[i].SpDesc = "None"
        tCARDS[i].SpVal = 0
        tCARDS[i].Copies = 1
        tCARDS[i].r = 63
        tCARDS[i].g = 42
        tCARDS[i].b = 20
        tCARDS[i].Wounds = 0
        tCARDS[i].FaceUp = false
      elseif string.find(line, "Name=") then tCARDS[i].Name = string.sub(line,6)
      elseif string.find(line, "Class=") then tCARDS[i].Class = string.sub(line,7) 
      elseif string.find(line, "Habitat=") then
        tCARDS[i].Habitat = string.sub(line,9)
        if love.filesystem.getInfo("habitats/"..tCARDS[i].Habitat..".png", "file") then
          tCARDS[i].Front = love.graphics.newImage("habitats/"..tCARDS[i].Habitat..".png")
        end
      elseif string.find(line, "Tag=") then tCARDS[i].Tag = string.sub(line,5) 
      elseif string.find(line, "Type=") then tCARDS[i].Type = string.sub(line,6) 
      elseif string.find(line, "Pic=") then 
        j = math.floor(tonumber(string.sub(line,5)))
        if j > 0 then
          if j<10 then s = "images/pic00"..j..".jpg"
          elseif j<100 then s = "images/pic0"..j..".jpg"
          else s = "images/pic"..j..".jpg" end
          if love.filesystem.getInfo(s, "file") then tCARDS[i].Image = love.graphics.newImage(s) end
        end
      elseif string.find(line, "Cost=") then tCARDS[i].Cost = tonumber(string.sub(line,6)) 
      elseif string.find(line, "MaxHealth=") then tCARDS[i].MaxHealth = tonumber(string.sub(line,11)) 
      elseif string.find(line, "Power=") then tCARDS[i].Power = tonumber(string.sub(line,7))
      elseif string.find(line, "Special=") then tCARDS[i].Special = string.sub(line,9)
      elseif string.find(line, "SpDesc=") then tCARDS[i].SpDesc = string.sub(line,8)
      elseif string.find(line, "SpVal=") then tCARDS[i].SpVal = tonumber(string.sub(line,7))
      elseif string.find(line, "Copies=") then tCARDS[i].Copies = tonumber(string.sub(line,8))
      else end
    end
  end
end

function love.update()
  if gFADE > 0 then  -- fade in graphics
    gFADE = gFADE - gFADESPEED
    if gFADE < 0 then gFADE = 0 end
  end
  gFLASH = gFLASH + gFLASH_z  -- flash counter
  if gFLASH > 255 then  -- this affects everything that flashes (same speed)
    gFLASH = gFLASH_max
    gFLASH_z = 0 - gFLASH_z
  elseif gFLASH < gFLASH_min then
    gFLASH = gFLASH_min
    gFLASH_z = 0 - gFLASH_z
  else end
  if love.timer.getTime() - gTIMER <= (1/gFPS) then love.timer.sleep((1/gFPS)-(love.timer.getTime()-gTIMER)) end  -- frame rate
  gTIMER = love.timer.getTime()  -- new timer
end

function love.draw()
  love.graphics.scale(gRATIOX, gRATIOY)
  if gGAMEMODE == "Title Screen" then drawtitle()
  elseif gGAMEMODE == "Main Menu" then drawmenu()
  elseif gGAMEMODE == "Level Select" then drawselectlevel()
  elseif gGAMEMODE == "Hero Select" then drawselecthero()
  elseif gGAMEMODE == "Deck Select" then drawselectdeck()  
  elseif gGAMEMODE == "Shop" then drawshop()
  elseif gGAMEMODE == "Battle" then drawbattle()
  else end
  if gFADE > 0 then 
    love.graphics.setColor(0, 0, 0, gFADE)
    love.graphics.rectangle("fill", 0, 0, 800, 600)
  else end  
end

function drawmenu()
  local menu_text = {"Single Player", "Shop", "Pass and Play", "Exit"}
  local menu_x = {520, 410, 530, 400}
  local menu_y = {200, 275, 350, 425}
  local icon_y = {190, 265, 340, 415}
  local x = love.mouse.getX() / gRATIOX
  local y = love.mouse.getY() / gRATIOY
  local i
  --love.graphics.setFont(fTITLE)
  --love.graphics.setColor(255, 255, 255, 255)
  --love.graphics.printf("Main Menu", 200, 50, 400, "center")
  --love.graphics.setFont(fHEADING)
  --for i = 1, 4, 1 do
  --  love.graphics.setColor(255, 255, 255, 255)
  --  if x>=350 and x<=menu_x[i] and y>=menu_y[i] and y<=menu_y[i]+25 then love.graphics.setColor(255, 255, 255, gFLASH) end
  --  if x>=280 and x<=320 and y>=icon_y[i]+5 and y<=icon_y[i]+45 then love.graphics.setColor(255, 255, 255, gFLASH) end
  --  love.graphics.printf(menu_text[i], 350, menu_y[i], 200, "left")
  --  love.graphics.draw(iICON[i+4], 275, icon_y[i])
  --end
  --love.graphics.setColor(255, 255, 255, 255)
  --love.graphics.printf("Sounds", 25, 500, 100, "center")
  --love.graphics.printf("Music", 675, 500, 100, "center")
  --if gMUSICON == true then love.graphics.draw(iICON[2], 700, 525)
  --else love.graphics.draw(iICON[1], 700, 525) end
  --if gSOUNDON == true then love.graphics.draw(iICON[4], 50, 525)
  --else love.graphics.draw(iICON[3], 50, 525) end
  drawcard(tCARDS[1], 10, 10, 150, 210, 10)
  drawcard(tCARDS[1], 200, 10, 300, 420, 10)
  drawcard(tCARDS[1], 10, 300, 75, 105, 10)
  drawcard(tCARDS[1], 90, 300, 50, 70, 10)
  drawcard(tCARDS[1], 150, 300, 30, 42, 10)
end

function drawcard(card, x, y, width, height, angle)
  local h = gethealth(card)
  local c = getcost(card)
  local p = getpower(card)
  local r, g, b = card.r, card.g, card.b
  local s = ""
  if c == 0 and card.Type == "hero" then c = "H" end
  angle = angle * math.pi / 180
  love.graphics.setColor(255, 255, 255, 255)                                                                                  -- White
  love.graphics.draw(card.Front, x, y, angle, width/300, height/420, 0, 0)                                -- Front of card
  love.graphics.draw(card.Image, x+(width*0.233), y+(height*0.136), angle, width/300, height/420, 0, 0)  -- Card picture
  if p > 0 then love.graphics.draw(iICON[9], x+(width*0.77), y+(height*0.82), angle, width/300, height/420, 0, 0) end  -- Power pawprint image
  love.graphics.draw(iICON[11], x+(width*0.013), y+(height*0.01), angle, width/300, height/420, 0, 0)                  -- Coin (cost) image
  
  love.graphics.setFont(fTITLE)
  love.graphics.setColor(0, 0, 0, 255)
  if p > 0 then love.graphics.printf(p, x+(width*0.793), y+(height*0.886), 60, "center", angle, width/300, height/420, 0, 0) end  --  Power text (black)
  love.graphics.printf(c, x+(width*0.02), y+(height*0.045), 60, "center", angle, width/300, height/420, 0, 0)                     -- Cost of card (black)
  love.graphics.setColor(255, 255, 255, 255)
  if p > 0 then love.graphics.printf(p, x+(width*0.787), y+(height*0.88), 60, "center", angle, width/300, height/420, 0, 0) end    --  Power text (white)
  love.graphics.printf(c, x+(width*0.013), y+(height*0.04), 60, "center", angle, width/300, height/420, 0, 0)                      -- Cost of card (white)

  love.graphics.setFont(fHEADING)
  love.graphics.setColor(0, 0, 0, 255)
  love.graphics.printf(card.Name, x+(width*0.247), y+(height*0.04), 215, "left", angle, width/300, height/420, 0, 0)              -- Card name (black)
  love.graphics.printf(card.Class, x+(width*0.247), y+(height*0.855), 160, "center", angle, width/300, height/420, 0, 0)          -- Card class (black)
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.printf(card.Name, x+(width*0.24), y+(height*0.035), 215, "left", angle, width/300, height/420, 0, 0)              -- Card name (white)
  love.graphics.printf(card.Class, x+(width*0.24), y+(height*0.85), 160, "center", angle, width/300, height/420, 0, 0)            -- Card class (white)

  if h > 0 then
    if h > card.MaxHealth*0.7 then love.graphics.setColor(0, 255, 0, 255)                                               -- Green heart (healthy)
    elseif h > card.MaxHealth*0.4 then love.graphics.setColor(255, 140, 0, 255)                                         -- Orange heart (damaged)
    else love.graphics.setColor(255, 0, 0, 255) end                                                                     -- Red heart (near dead)
    love.graphics.draw(iICON[10], x, y+(height*0.85), angle, width/300, height/420, 0, 0)                                         -- Heart icon
    love.graphics.setColor(0, 0, 0, 255)
    love.graphics.printf(h, x+(width*0.02), y+(height*0.864), 30, "center", angle, width/300, height/420, 0, 0)                   -- Health text (black)
    love.graphics.printf("/", x+(width*0.017), y+(height*0.883), 60, "center", angle, width/300, height/420, 0, 0)
    love.graphics.printf(card.MaxHealth, x+(width*0.12), y+(height*0.905), 30, "center", angle, width/300, height/420, 0, 0)
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.printf(h, x+(width*0.013), y+(height*0.86), 30, "center", angle, width/300, height/420, 0, 0)                   -- Health text (white)
    love.graphics.printf("/", x+(width*0.013), y+(height*0.88), 60, "center", angle, width/300, height/420, 0, 0)
    love.graphics.printf(card.MaxHealth, x+(width*0.113), y+(height*0.9), 30, "center", angle, width/300, height/420, 0, 0)
  end
  
  if card.Special ~= "None" then                                                                                        -- Special text sentence
    s = card.Special..": "..card.SpDesc
    if card.SpVal ~= 0 then s = s.." Costs "..card.SpVal.."."
    else s = s.." Automatic." end
  end
  
  love.graphics.setFont(fDEFAULT)
  love.graphics.setColor(0, 0, 0, 255)
  love.graphics.printf(card.Tag, x+(width*0.05), y+(height*0.552), 270, "left", angle, width/300, height/420, 0, 0)               -- Card tag description (black)
  if s ~= "" then love.graphics.printf(s, x+(width*0.05), y+(height*0.712), 276, "left", angle, width/300, height/420, 0, 0) end  -- Special text (black)
  love.graphics.printf(card.Habitat, x+(width*0.247), y+(height*0.935), 160, "center", angle, width/300, height/420, 0, 0)         -- Habitat (black)
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.printf(card.Tag, x+(width*0.045), y+(height*0.55), 270, "left", angle, width/300, height/420, 0, 0)               -- Card tag description (white)
  if s ~= "" then love.graphics.printf(s, x+(width*0.045), y+(height*0.71), 276, "left", angle, width/300, height/420, 0, 0) end  -- Special text (white)
  love.graphics.printf(card.Habitat, x+(width*0.24), y+(height*0.93), 160, "center", angle, width/300, height/420, 0, 0)            -- Habitat (black)
end

function love.mousepressed(x, y, button, istouch)
  x = x / gRATIOX
  y = y / gRATIOY
  if gGAMEMODE == "Main Menu" then clickmenu(x, y) end
end

function clickmenu(x, y)
  if x>=350 and x<=520 and y>=200 and y<=225 then clicksingleplayer()
  elseif x>=350 and x<=410 and y>=275 and y<=300 then clickshop()
  elseif x>=350 and x<=530 and y>=350 and y<=375 then clickmultiplayer()
  elseif x>=350 and x<=400 and y>=425 and y<=450 then clickexit()
  elseif x>=280 and x<=320 and y>=195 and y<=235 then clicksingleplayer()
  elseif x>=280 and x<=320 and y>=270 and y<=310 then clickshop()
  elseif x>=280 and x<=320 and y>=345 and y<=385 then clickmultiplayer()
  elseif x>=280 and x<=320 and y>=420 and y<=460 then clickexit()
  elseif x>=55 and x<=95 and y>=530 and y<=570 then
    if gSOUNDON == true then gSOUNDON = false else gSOUNDON = true end
  elseif x>=705 and x<=745 and y>=530 and y<=570 then
    if gMUSICON == true then gMUSICON = false else gMUSICON = true end
  else end
end

function clicksingleplayer()
  gFADE = 255
  tCONTROL = {"Player", "AI"}
  gGAMEMODE = "Level Select"
end

function clickshop()
  gFADE = 255
  gGAMEMODE = "Shop"
end

function clickmultiplayer()
  gFADE = 255
  tCONTROL = {"Player", "Player"}
  gGAMEMODE = "Hero Select"
  
end

function clickexit()
  love.timer.sleep(0.5)
  love.event.quit()
end

function gethealth(card)
  return card.MaxHealth - card.Wounds
end

function getcost(card)
  return card.Cost
end

function getpower(card)
  return card.Power
end